#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os
import zipfile
from asyncio import as_completed
from concurrent.futures import ThreadPoolExecutor

import pandas as pd

import wtai_dt.YawAnalysis.Yaw


class YawAnalysis:
    def __init__(self,directory_path, trubine_field,hydraulic_pressure = None,angle_of_wind=None,fan_status=None,
                 yaw_require=None,yaw_times=None,build_pressure_speed=None,angle=None):
        """
            初始化偏航分析实例。
            :param directory_path: 数据文件的目录路径
            :param trubine_field: 风力发电机的字段名
            :param hydraulic_pressure: 液压压力，默认为None
            :param angle_of_wind: 风向角度，默认为None
            :param fan_status: 风机当前状态值，默认为None
            :param yaw_require: 偏航要求值，默认为None
            :param yaw_times: 偏航次数，默认为None
            :param build_pressure_speed: 建压速度，默认为None
            :param angle: 偏航角度，默认为None

            通过多线程分析路径下文件的偏航数据
        """
        self.directory_path = directory_path
        self.hydraulic_pressure = hydraulic_pressure
        self.angle_of_wind = angle_of_wind
        self.fan_status = fan_status
        self.yaw_require = yaw_require
        self.yaw_times = yaw_times
        self.build_pressure_speed = build_pressure_speed
        self.angle = angle
        self.trubine_field = trubine_field
        self.use_field = []
        if self.hydraulic_pressure is not None:
            self.use_field.append(hydraulic_pressure)
        if self.angle_of_wind is not None:
            self.use_field.append(angle_of_wind)
        if self.fan_status is not None:
            self.use_field.append(fan_status)
        if self.yaw_require is not None:
            self.use_field.append(yaw_require)
        if self.yaw_times is not None:
            self.use_field.append(yaw_times)
        if self.build_pressure_speed is not None:
            self.use_field.append(build_pressure_speed)
        if self.angle is not None:
            self.use_field.append(angle)
        if self.trubine_field is not None:
            self.use_field.append(trubine_field)

        self.Yaw = Yaw()

    def process_csv_datas(self, data,file_path):
        for i, d in data:
            if self.hydraulic_pressure:
                self.Yaw.maximum_hydraulic_pressure_average_value(d[self.hydraulic_pressure],i)
                self.Yaw.maximum_hydraulic_pressure(d[self.hydraulic_pressure],i)
            if self.angle_of_wind:
                self.Yaw.total_angle_of_wind_direction_change(d[self.angle_of_wind],i)
            if self.fan_status and self.yaw_require:
                self.Yaw.total_number_yawing_in_power_generation_state(d,self.fan_status,self.yaw_require,i)
                self.Yaw.maximum_yawing_time_in_power_generation_state(d,self.fan_status,self.yaw_require,i)
                self.Yaw.total_yaw_time_hours(d,self.fan_status,self.yaw_require,i)
            if self.yaw_times:
                self.Yaw.total_yaw_times(d[self.yaw_times], i)
            if self.build_pressure_speed:
                self.Yaw.maximum_build_pressure_speed(d[self.build_pressure_speed],i)
            if self.fan_status and self.yaw_require and self.angle:
                self.Yaw.maximum_slippage_angle_in_power_generation_state(d, self.fan_status, self.yaw_require, self.angle,i)
                self.Yaw.maximum_slippage_angle_in_non_power_generation_state(d, self.fan_status, self.yaw_require, self.angle,i)
            if self.yaw_require and self.angle:
                self.Yaw.total_angle_of_slippage_when_not_yawing(d,self.yaw_require,self.angle, i)

    def process_csv_data(self,file, name):
        encodings = ['gbk', 'utf-8', 'latin1']
        for encoding in encodings:
            try:
                data = pd.read_csv(io.StringIO(file.decode(encoding)), usecols=self.use_field,index_col=False)
                data = data.dropna()
                data = data.groupby(self.trubine_field)
                self.process_csv_datas(data,name)
                return
            except UnicodeDecodeError as e:
                print(f"Error processing file {name} with {encoding}: {e}")
            except Exception as e:
                print(f"Unexpected error processing file {name}: {e}")
                return
        print(f"All encodings failed for file {name}. Consider checking file encoding.")

    def extract_and_process_zip(self,zip_path, executor):
        try:
            with zipfile.ZipFile(zip_path, 'r') as z:
                for entry in z.infolist():
                    filename = entry.filename.encode('cp437').decode('gbk')
                    print("Processing entry with name:", filename)
                    if entry.filename.endswith('.zip'):
                        inner_zip_path = os.path.join('/tmp', os.path.basename(entry.filename))
                        with z.open(entry) as inner_file:
                            with open(inner_zip_path, 'wb') as f:
                                f.write(inner_file.read())
                        executor.submit(self.extract_and_process_zip, inner_zip_path, executor)
                    elif entry.filename.endswith('.csv'):
                        with z.open(entry) as file:
                            file_content = file.read()
                            self.process_csv_data(file_content, file.name)
        except zipfile.BadZipFile:
            print(f"Cannot open ZIP file as it is either a bad zip file or corrupted: {zip_path}")
        except Exception as e:
            print(f"General Error handling ZIP file {zip_path}: {e}")

    # 偏航分析入口程序
    def analysis(self):
        with ThreadPoolExecutor(max_workers=32) as executor:
            futures = []
            for root, dirs, files in os.walk(self.directory_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    if file_path.endswith('.zip'):
                        futures.append(executor.submit(self.extract_and_process_zip, file_path, executor))
                    elif file_path.endswith('.csv'):
                        with open(file_path, 'rb') as f:
                            file_content = f.read()
                            self.process_csv_data(file_content, file_path)
            for future in as_completed(futures):
                print("Completed processing for a future.")

    def get_result(self):
        return self.Yaw.get_result()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from typing import Optional, Dict
import pandas as pd

class Yaw:
    def __init__(self):
        self.result_dict: Dict[str, float] = {}

    def get_result(self) -> Dict[str, float]:
        return self.result_dict

    def maximum_hydraulic_pressure_average_value(
            self, data: Optional[pd.Series], id: Optional[str]
    ) -> None:
        """
            计算最大液压压力平均值
            参数:
            data (pd.Series): 液压压力数据
            id (str, optional): 当前计算风机ID

            计算一组数据中,最大液压压力的平均值
        """
        t = data.mean().item()
        t = round(t, 3)
        if f"{id} 最大液压压力平均值(Pa)" not in self.result_dict:
            self.result_dict[f"{id} 最大液压压力平均值(Pa)"] = t
        else:
            self.result_dict[f"{id} 最大液压压力平均值(Pa)"] = round((t + self.result_dict[
                f"{id} 最大液压压力平均值(Pa)"]) / 2.0, 3)

    def maximum_hydraulic_pressure(
            self, data: Optional[pd.Series], id: Optional[str]
    ) -> None:
        """
            计算最大液压压力
            参数:
            data (pd.Series): 液压压力数据
            id (str, optional): 当前计算风机ID

            计算风电机组最大液压压力
        """
        t = data.max().item()
        if f"{id} 最大液压压力(Pa)" not in self.result_dict:
            self.result_dict[f"{id} 最大液压压力(Pa)"] = t
        else:
            self.result_dict[f"{id} 最大液压压力(Pa)"] = t if t > self.result_dict[f"{id} 最大液压压力(Pa)"] else self.result_dict[
                f"{id} 最大液压压力(Pa)"]

    def total_angle_of_wind_direction_change(
            self, data: Optional[pd.Series] , id: Optional[str]
    ) -> None:
        """
            计算风机风向变化总角度
            参数:
            data (pd.Series): 风向角度数据
            id (str, optional): 计算风机ID
        """
        t = data.dropna().diff().abs().sum().item()
        if f"{id} 风向变化总角度(°)" not in self.result_dict:
            self.result_dict[f"{id} 风向变化总角度(°)"] = t
        else:
            self.result_dict[f"{id} 风向变化总角度(°)"] += t

    def total_number_yawing_in_power_generation_state(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str] , id: Optional[str] = None
    ) -> None:
        """
            发电状态下,偏航总次数

            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 风机状态值.
            field2 (str): 偏航要求值.
            id (str, optional): 计算风机ID.

            计算风机状态值为6且偏航要求值置1或2的次数之和.
        """
        condition_field1 = (data[field1] == 6)
        field2_series = data[field2][condition_field1]
        is_zero = field2_series == 0
        shifted_is_zero = is_zero.shift(1).fillna(True)
        change_to_non_zero = field2_series.isin([1, 2]) & shifted_is_zero
        total_count = change_to_non_zero.sum()
        key = f"{id} 发电状态偏航总次数"
        self.result_dict[key] = self.result_dict.get(key, 0) + total_count

    def maximum_yawing_time_in_power_generation_state(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str] , id: Optional[str] = None
    ) -> None:
        """
            风机发电状态最大偏航时长
            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 风机状态值.
            field2 (str): 偏航要求值.
            id (str, optional): 计算风机ID.

            风机状态值为6，偏航要求值为1或2，单次偏航最大时间.对于一组数据,单次偏航时间为风机状态值为6，偏航要求值为1或2连续的数据量
        """
        mask = (data[field1] == 6) & (
                (data[field2] == 1) | (data[field2] == 2))
        groups = (mask != mask.shift()).cumsum()
        durations = data[mask].groupby(groups).size()
        max_duration = durations.max() if not durations.empty else 0
        key = f"{id} 发电状态最大偏航时长(秒)"
        self.result_dict[key] = max(self.result_dict.get(key, 0), max_duration)
        mask = data[data[field1] == 6]
        if f"{id} 有效数据量" not in self.result_dict:
            self.result_dict[f"{id} 有效数据量"] = len(mask)
        else:
            self.result_dict[f"{id} 有效数据量"] += len(mask)

    def total_yaw_times(
            self, data: Optional[pd.Series] , id: Optional[str]
    ) -> None:
        """
            总偏航次数
            参数:
            data (pd.Series): 风机数据.
            id (str, optional): 计算风机ID.

            风机总偏航次数
        """
        t = round((data.iloc[-1] - data.iloc[0]).item(), 0)
        if f"{id} 总偏航次数" not in self.result_dict:
            self.result_dict[f"{id} 总偏航次数"] = t
        else:
            self.result_dict[f"{id} 总偏航次数"] += t

    # 总偏航时间
    def total_yaw_time_hours(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str] , id: Optional[str] = None
    ) -> None:
        """
            风机总偏航时间
            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 风机状态值.
            field2 (str): 偏航要求值.
            id (str, optional): 计算风机ID.

            风机状态值为6，偏航要求值为1或2，单次偏航最大时间.对于一组数据,单次偏航时间为风机状态值为6，偏航要求值为1或2连续的数据量
        """
        filtered_data = data[(data[field1] == 6) & (
                (data[field2] == 1) | (data[field2] == 2))]
        sum_time = round(1.0 * len(filtered_data) / 3600, 3)
        if f"{id} 总偏航时间(小时)" not in self.result_dict:
            self.result_dict[f"{id} 总偏航时间(小时)"] = round(sum_time, 3)
        else:
            self.result_dict[f"{id} 总偏航时间(小时)"] += round(sum_time, 3)

    # 最大建压速度
    def maximum_build_pressure_speed(
            self, data: Optional[pd.Series] , id: Optional[str]
    ) -> None:
        """
            最大建压速度
            参数:
            data (pd.Series): 风机建压速度数据.
            id (str, optional): 计算风机ID.
        """
        t = data.max().item()
        if f"{id} 最大建压速度" not in self.result_dict:
            self.result_dict[f"{id} 最大建压速度"] = t
        else:
            self.result_dict[f"{id} 最大建压速度"] = t if t > self.result_dict[
                f"{id} 最大建压速度"] else self.result_dict[f"{id} 最大建压速度"]

    def maximum_slippage_angle_in_power_generation_state(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str], field3: Optional[str], id: Optional[str] = None
    ) -> None:
        """
            发电状态下滑移角度最大值
            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 风机状态值.
            field2 (str): 偏航要求值.
            field3 (str): 偏航角度.
            id (str, optional): 计算风机ID.

            风机状态值为6，偏航要求值为0，偏航角度大于0.013*360的偏航角度最大值
        """
        data = data.dropna()
        filtered_data = data[
            (data[field1] == 6) &
            (data[field2] == 0)
        ]
        angle_field = field3
        filtered_data = filtered_data[filtered_data[angle_field].diff().abs() > 0.013]
        angle = filtered_data[angle_field].diff().abs() * 360
        if angle.empty:
            return
        else:
            t = round(angle.max().item(), 3)
            if f"{id} 发电状态下的滑移角度最大值(°)" not in self.result_dict:
                self.result_dict[f"{id} 发电状态下的滑移角度最大值(°)"] = t
            else:
                self.result_dict[f"{id} 发电状态下的滑移角度最大值(°)"] = t if t > self.result_dict[
                    f"{id} 发电状态下的滑移角度最大值(°)"] else self.result_dict[f"{id} 发电状态下的滑移角度最大值(°)"]

    def maximum_slippage_angle_in_non_power_generation_state(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str], field3: Optional[str], id: Optional[str] = None
    ) -> None:
        """
            非发电状态下的滑移角度最大值
            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 风机状态值.
            field2 (str): 偏航要求值.
            field3 (str): 偏航角度.
            id (str, optional): 计算风机ID.

            风机状态值不为6，偏航要求值为0，偏航角度大于0.013*360的偏航角度最大值
        """
        data = data.dropna()
        filtered_data = data[
            (data[field1] != 6) &
            (data[field2] == 0)
            ]
        # 0.013
        angle_field = field3
        filtered_data = filtered_data[filtered_data[angle_field].diff().abs() > 0.013]
        angle = filtered_data[angle_field].diff().abs() * 360
        if angle.empty:
            return  # 如果 angle 为空，则返回 None 或其他适当的值
        else:
            t = int(angle.max().item())
            if f"{id} 非发电状态下的滑移角度最大值(°)" not in self.result_dict:
                self.result_dict[f"{id} 非发电状态下的滑移角度最大值(°)"] = t
            else:
                self.result_dict[f"{id} 非发电状态下的滑移角度最大值(°)"] = t if t > self.result_dict[
                    f"{id} 非发电状态下的滑移角度最大值(°)"] else self.result_dict[f"{id} 非发电状态下的滑移角度最大值(°)"]

    def total_angle_of_slippage_when_not_yawing(
            self, data: Optional[pd.DataFrame], field1: Optional[str], field2: Optional[str], id: Optional[str] = None
    ) -> None:
        """
            未偏航时滑移总角度
            参数:
            data (pd.DataFrame): 风机数据.
            field1 (str): 偏航要求值.
            field2 (str): 偏航角度.
            id (str, optional): 计算风机ID.

            偏航要求值为0，偏航角度大于0.013*360的偏航角度累加
        """
        filtered_data = data[(data[field1] == 0)]
        angle_field = field2
        filtered_data = filtered_data[filtered_data[angle_field].diff().abs() > 0.013]
        twisted_cable_change = filtered_data[field2].diff().abs().sum()
        total_sum = twisted_cable_change * 360
        total_sum = round(total_sum, 0)
        key = f"{id} 未偏航时滑移总角度(°)"
        self.result_dict[key] = self.result_dict.get(key, 0) + total_sum



from moviepy.editor import *
import json
import argparse
import sys
import os
print("Importing wtai_dt.py")
# wtai_dt(wind turbine ai and digital twin)
# “数据统计 报告 绘图
# wtai_dt.datadescribe.function
# wtai_dt.report.function
# wtai_dt.plot.function
# wtai_dt.bujian.function
# ”
'''
简单地剪辑视频文件！

请在当前目录下创建一个json文件，文件名为to_erase.json，内容为要去掉的时间片段，例如：
[   ["0:22", "1:14"],
    ["1:25", "1:30"],
    ["2:13", "2:25"]
]

然后准备一个视频文件，例如full.mp4，运行：
```python
python wtai_dt.py
```

最后得到的视频文件为final.mp4
'''
def run(input_file='full.mp4', output_file='final.mp4', json_file="to_erase.json"):
    #读取参数
    parser = argparse.ArgumentParser(description="Cut video by moviepy!\n \
        Please create a json file named to_erase.json in the current directory, \
        the content is the time segments to erase, for example:\n \
        [   [\"0:22\", \"1:14\"],\n \
            [\"1:25\", \"1:30\"],\n \
            [\"2:13\", \"2:25\"]\n \
        ]\n \
        Then prepare a video file, for example: full.mp4\n\
    ")
    parser.add_argument("--input", default='full.mp4', help="input video file")
    parser.add_argument("--output", default='final.mp4', help="output video file")
    input_file = parser.parse_args().input
    output_file = parser.parse_args().output

    # 设定要去掉的时间片段
    json_file = "to_erase.json"
    erase = None
    with open(json_file, "r") as f:
        erase = json.load(f)
    print("Erase these clips: ", erase)

    # 将要去掉时间片段转换为要保留的时间片段
    def erase_to_keep(full, erase):
        flat_list = [item for sublist in erase for item in sublist]
        flat_list.insert(0, 0)
        flat_list.append(full.duration)
        keep = []
        for i in range(0, len(flat_list), 2):
            keep.append([flat_list[i], flat_list[i+1]])
        return keep

    # 读取视频文件
    full = VideoFileClip(input_file)
    # 转换为要保留的时间片段
    keep = erase_to_keep(full, erase)
    print("Clips to keep:", keep)


    # 利用subclip将要保留的时间片段剪切出来
    def keep_some_clip(full, keep):
        clips = []
        for i, cut in enumerate(keep):
            print(f"Cutting clip No.{i}, start {cut[0]}, end {cut[1]}")
            clip = full.subclip(cut[0], cut[1])
            clips.append(clip)
        return clips

    clips = keep_some_clip(full, keep)
    final = concatenate_videoclips(clips)
    final.write_videofile(output_file)

if __name__ == '__main__':
    run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False

class GeneratorPerformance:
    def __init__(self, data, filed1, filed2, xlabel, ylable, function):
        """
            初始化发电性能分析绘图类,转矩-转速曲线,转速-桨距角曲线,风速-桨距角曲线,风速-转速曲线,风速-功率曲线

            Args:
                data: 绘图数据
                filed1: X轴字段
                filed2: Y轴字段
                xlabel: X轴label
                ylabel: Y轴label
                function: 绘制的具体曲线
        """

        self.data = data
        self.filed1 = filed1
        self.filed2 = filed2
        self.function = function
        self.labels= [xlabel,ylable]

    def Plot(self):
        self.ScatterPlot(self.data[self.filed1],self.data[self.filed2],self.function,self.labels)

    def ScatterPlot(self, x, y, title, labels, is_wind=False, wind_data=None):
        plt.close()
        plt.scatter(x, y, s=1, alpha=0.5)
        if is_wind:
            wind_data_float = {float(k): float(v) for k, v in wind_data.items()}
            plt.plot(list(wind_data_float.keys()), list(wind_data_float.values()))
        plt.xlabel(labels[0])
        plt.ylabel(labels[1])
        plt.gca().spines['right'].set_visible(False)
        plt.gca().spines['top'].set_visible(False)
        plt.title(title)
        if "转矩-转速曲线" in title:
            plt.xlim(xmin=0, xmax=1600)
        elif "转速-桨距角曲线" in title or "风速-桨距角曲线" in title:
            plt.xlim(xmin=0, xmax=25)
            plt.ylim(ymin=-5, ymax=30)
        elif "风速-转速曲线" in title:
            plt.xlim(xmin=0, xmax=25)
        else:
            plt.ylim(ymin=y.min(), ymax=float(y.max()) * 1.3)
        plt.show()
        plt.close()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re

import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
colors = plt.cm.tab20.colors


def wind_error_image(data, group_one, group_two, x_label, y_label, title, image_name, types=None):
    plt.close()
    plt.figure(figsize=(12, 6))

    if types:
        use_data = data[data['types'] == types]
    else:
        use_data = data

    if use_data.empty:
        plt.bar([], [])
        plt.xlabel(x_label)
        plt.ylabel(y_label)
        plt.title(title)
        plt.xticks(rotation=90)
        plt.tight_layout()
        plt.show()
        return

    grouped = use_data.groupby([group_one, group_two])['count'].agg('sum').unstack().fillna(0)
    error_types = grouped.columns

    bottom = np.zeros(len(grouped))
    for i, error_type in enumerate(error_types):
        color = colors[i % len(colors)]
        plt.bar(grouped.index, grouped[error_type], bottom=bottom, label=error_type, color=color)
        bottom += grouped[error_type].values

    plt.xlabel(x_label)
    plt.ylabel(y_label)
    plt.title(title)
    plt.legend(title='故障类型', loc='upper center', bbox_to_anchor=(0.5, -0.15), ncol=len(error_types))
    plt.xticks(rotation=90)
    plt.tight_layout()
    plt.show()
    plt.close()

class SCADAError:
    def __init__(self, data, wind_field, start_time_field, end_time_field, error_type_field, function):
        """
        分析SCADA系统故障数据

        Args:
            data: 故障数据
            wind_field: 风机字段
            start_time_field: 故障开始时间字段
            end_time_field: 故障结束时间字段
            error_type_field: 故障类型字段
            function: 故障分析方法:故障次数,停机时间
        """

        self.data = data
        self.wind_field = wind_field
        self.start_time_field = start_time_field
        self.end_time_field = end_time_field
        self.error_type_field = error_type_field
        self.function = function
        self.result = {}
        self.result_dict = {}
        self.init_data()

    def init_data(self):
        self.data[self.start_time_field] = pd.to_datetime(self.data[self.start_time_field], errors='coerce')
        self.data[self.end_time_field] = pd.to_datetime(self.data[self.end_time_field], errors='coerce')

    def error_time(self, data, start_time, end_time, error_status):
        data = data[data[error_status].str.contains('故障')]
        df = data.sort_values(by=[start_time])

        def is_overlapping(start1, end1, start2, end2):
            tolerance = pd.Timedelta(minutes=30)
            return end1 + tolerance >= start2

        def merge_intervals(intervals):
            if not intervals:
                return []
            sorted_intervals = sorted(intervals, key=lambda x: x[0])
            merged = [sorted_intervals[0]]
            for current in sorted_intervals[1:]:
                previous = merged[-1]
                if is_overlapping(previous[0], previous[1], current[0], current[1]):
                    merged[-1] = (previous[0], max(previous[1], current[1]), previous[2])
                else:
                    merged.append(current)
            return merged

        intervals = list(
            zip(df[start_time], df[end_time], df[error_status]))
        merged_intervals = merge_intervals(intervals)
        fault_analysis = {
            'Total Faults': len(merged_intervals),
            'Fault Details': [{'Start': start, 'End': end, 'Fault Code': code, 'Duration': end - start} for
                              start, end, code
                              in merged_intervals],
            'Total Duration': sum((end - start).total_seconds() for start, end, code in merged_intervals)
        }
        return fault_analysis

    def down_time(self, data, error_status, start_time, end_time):
        data = data[data[error_status].str.contains('故障')]
        df = data.sort_values(by=[start_time])

        def is_overlapping(start1, end1, start2, end2):
            return max(start1, start2) <= min(end1, end2)

        def merge_intervals(intervals):
            if not intervals:
                return []

            sorted_intervals = sorted(intervals, key=lambda x: x[0])

            merged = [sorted_intervals[0]]
            for current in sorted_intervals:
                previous = merged[-1]
                if is_overlapping(previous[0], previous[1], current[0], current[1]):
                    merged[-1] = (previous[0], max(previous[1], current[1]), previous[2])
                else:
                    merged.append(current)

            return merged

        intervals = list(
            zip(df[start_time], df[end_time], df[error_status]))
        merged_intervals = merge_intervals(intervals)
        fault_analysis = {
            'Total Faults': len(merged_intervals),
            'Fault Details': [{'Start': start, 'End': end, 'Fault Code': code, 'Duration': end - start} for
                              start, end, code in merged_intervals]
        }
        return fault_analysis['Total Faults']

    def error_analysis(self):
        try:
            datas = self.data.groupby(self.wind_field)
            rows = []
            fault_analysis = {}
            for index, data in datas:
                data = data.reset_index(drop=True)
                use_result = self.error_time(data, self.start_time_field,
                                        self.end_time_field,
                                        self.error_type_field)
                fault_analysis[index] = {
                    'Total Faults': use_result['Total Faults'],
                    'Fault Details': use_result['Fault Details']
                }
                for detail in use_result['Fault Details']:
                    rows.append({
                        'Unit': index,
                        'Start': detail['Start'],
                        'End': detail['End'],
                        'Fault Code': detail['Fault Code'],
                        'Duration': detail['Duration']
                    })
                if self.function == '故障次数':
                    key = str(index) + ' 故障次数'
                    self.result[key] = use_result['Total Faults']
                    print(key + str(self.result[key]))
                else:
                    key = str(index) + ' 停机时间'
                    self.result[key] = str(use_result['Total Duration']) + ' (秒)'
                    print(key + str(self.result[key]))
            result_df = pd.DataFrame(rows)
            result_df.to_csv(('fault_analysis_results.csv'), index=False)
            return fault_analysis
        except Exception as e:
            print(e)
            return False

    def error_statistical(self,_type="故障"):
        """
        分析SCADA系统故障类型
        Args:
            _type:故障、警告
        """
        self.error_type = _type
        result = self.error_analysis()
        for product, info in result.items():
            res = self.type_statistics(info, _type)
            if product +'_'+ _type not in self.result_dict:
                self.result_dict[product +'_'+ _type] = res
            else:
                for key in self.result_dict[product + _type].keys():
                    self.result_dict[product +'_'+ _type][key] = [a + b for a, b in zip(self.result_dict[product +'_'+ _type][key], res[key])]
        self.pie_image(self.result_dict)

    def pie_image(self, data):
        data_list = []
        for product, faults in data.items():
            for category, values in faults.items():
                count, duration = values
                unit, types = product.split('_')
                data_list.append([unit, category, count, duration, types])
        data = pd.DataFrame(data_list, columns=['unit', 'error_type', 'count', 'duration', 'types'])
        wind_error_image(data, 'unit', 'error_type', '风机', '故障次数', '各风机故障类型统计（全部）', '各风机故障类型统计（全部）')
        wind_error_image(data, 'unit', 'error_type', '风机', '故障次数', '各风机故障类型统计（故障）', '各风机故障类型统计（故障）',types='故障')
        wind_error_image(data, 'unit', 'error_type', '风机', '故障次数', '各风机故障类型统计（报警）', '各风机故障类型统计（报警）',types='报警')
        wind_error_image(data, 'error_type', 'types', '故障类型', '故障次数', '故障类型分类统计', '故障类型分类统计')
        wind_error_image(data, 'unit', 'types', '风机', '故障次数', '各风机故障类型分类统计', '各风机故障类型分类统计')
        types = data['error_type'].unique()
        for tp in types:
            plt.figure(figsize=(12, 6))
            use_data = data[data['error_type'] == tp]
            wind_error_image(use_data, 'unit', 'types', '风机', '故障次数', f'各风机{tp}统计', f'各风机{tp}统计')

    def type_statistics(self, info, _type="故障"):
        example = {
            '变流器故障': [0, 0],
            '变桨故障': [0, 0],
            '偏航故障': [0, 0],
            '主控故障': [0, 0],
            '发电机故障': [0, 0],
            '润滑故障': [0, 0],
            '水冷故障': [0, 0],
            '其他故障': [0, 0],
        }
        for fault in info['Fault Details']:
            fault['Fault Code'] = int(re.findall(r'\d+', fault['Fault Code'])[0])
            print(
                f" - Start: {fault['Start']}, End: {fault['End']}, Fault Code: {fault['Fault Code']}, Duration: {fault['Duration']}")
            if _type == "故障":
                if fault['Fault Code'] in range(17, 30 + 1) or fault['Fault Code'] in range(401, 512 + 1) or fault[
                    'Fault Code'] == 660 or fault['Fault Code'] in range(794, 937 + 1):
                    example['变流器故障'][0] += 1
                    example['变流器故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] in range(304, 383 + 1) or fault['Fault Code'] in range(387, 399 + 1) or fault[
                    'Fault Code'] in range(514, 519 + 1) or fault['Fault Code'] in range(
                    601, 615 + 1) or fault['Fault Code'] in range(661, 663 + 1) or fault['Fault Code'] in range(701,
                                                                                                                793 + 1):
                    example['变桨故障'][0] += 1
                    example['变桨故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] == 210 or fault['Fault Code'] in range(223, 233 + 1) or fault[
                    'Fault Code'] in range(260, 261 + 1) or fault['Fault Code'] in range(290, 292 + 1):
                    example['偏航故障'][0] += 1
                    example['偏航故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] == 1 or fault['Fault Code'] == 7 or fault['Fault Code'] in range(9, 14 + 1) or \
                        fault['Fault Code'] in range(30, 47 + 1) or fault['Fault Code'] in range(
                    211, 222 + 1) or fault['Fault Code'] in range(234, 239 + 1) or fault['Fault Code'] in range(293,
                                                                                                                297 + 1):
                    example['主控故障'][0] += 1
                    example['主控故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] in range(240, 255 + 1) or fault['Fault Code'] in range(262, 271 + 1):
                    example['发电机故障'][0] += 1
                    example['发电机故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] in range(275, 279 + 1) or fault['Fault Code'] in range(384, 386 + 1):
                    example['润滑故障'][0] += 1
                    example['润滑故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] in range(100, 111 + 1) or fault['Fault Code'] in range(150, 161 + 1):
                    example['水冷故障'][0] += 1
                    example['水冷故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] == 5 or fault['Fault Code'] == 112 or fault['Fault Code'] in range(130,
                                                                                                            131 + 1) or \
                        fault['Fault Code'] == 204 or fault['Fault Code'] in range(256, 258 + 1) or fault[
                    'Fault Code'] in range(
                    272, 274 + 1) or fault['Fault Code'] in range(280, 289 + 1):
                    example['其他故障'][0] += 1
                    example['其他故障'][1] += fault['Duration'].total_seconds()
                else:
                    print("未收录故障？？？")
            else:
                if fault['Fault Code'] in range(240, 255 + 1) or fault['Fault Code'] in range(262, 271 + 1):
                    example['发电机故障'][0] += 1
                    example['发电机故障'][1] += fault['Duration'].total_seconds()
                elif fault['Fault Code'] in range(275, 279 + 1) or fault['Fault Code'] in range(384, 386 + 1):
                    example['润滑故障'][0] += 1
                    example['润滑故障'][1] += fault['Duration'].total_seconds()
        print(example)
        return example


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from docx import Document
from docx.shared import Pt, Inches, RGBColor, Cm
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT, WD_ALIGN_PARAGRAPH
from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT
from docx.oxml import OxmlElement
from docx.oxml.ns import qn
class Report:
    def __init__(self, cover_path=None):
        self.cover_path = cover_path

    def create_new_document(self):
        if self.cover_path:
            return Document(self.cover_path)
        return Document()

    def insert_lettle_title_word(self, doc, lettle_text):
        subtitle = doc.add_heading(level=2)
        subtitle_run = subtitle.add_run(lettle_text)
        subtitle_run.font.bold = True
        subtitle_run.font.name = '宋体'
        subtitle_run.font.color.rgb = RGBColor(0, 0, 0)
        subtitle_run._element.rPr.rFonts.set(qn('w:eastAsia'), '宋体')
        subtitle_run.font.size = Pt(12)

    def insert_paragraph(self, doc, text, font_name=u'宋体', font_size=12):
        paragraph = doc.add_paragraph('\t' + text)
        for run in paragraph.runs:
            self.set_font(run, font_name, font_size)

    def insert_image_word(self, doc, image_path):
        section = doc.sections[-1]
        page_width = section.page_width
        left_margin = section.left_margin
        right_margin = section.right_margin
        available_width = page_width - left_margin - right_margin
        p = doc.add_paragraph()
        run = p.add_run()
        run.add_picture(image_path, width=available_width)

    def set_cell_properties(self, cell):
        for paragraph in cell.paragraphs:
            for run in paragraph.runs:
                run.font.size = Pt(10.5)
                run.font.color.rgb = RGBColor(0, 0, 0)
                run.font.name = '宋体'
                run._element.rPr.rFonts.set(qn('w:eastAsia'), '宋体')
        cell.paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        cell.vertical_alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

    def insert_table_word(self, doc, data):
        table = doc.add_table(rows=1, cols=len(data[0]))
        table.style = 'Table Grid'

        hdr_cells = table.rows[0].cells
        for i, header in enumerate(data[0]):
            hdr_cells[i].text = header
            self.set_cell_properties(hdr_cells[i])

        for row in data[1:]:
            row_cells = table.add_row().cells
            for i, value in enumerate(row):
                row_cells[i].text = value
                self.set_cell_properties(row_cells[i])

    def set_font(self, run, font_name=u'宋体', font_size=12):
        font = run.font
        font.name = font_name
        run._element.rPr.rFonts.set(qn('w:eastAsia'), font_name)
        font.size = Pt(font_size)

    def insert_centered_title(self,paragraph, title):
        paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
        run = paragraph.add_run(title)
        self.set_font(run)

    def insert_title(self, doc, title_text):
        title = doc.add_heading(level=1)
        title_run = title.add_run(title_text)
        title_run.font.bold = True
        title_run.font.name = '宋体'
        title_run.font.color.rgb = RGBColor(0, 0, 0)  # 设置为黑色字体
        title_run._element.rPr.rFonts.set(qn('w:eastAsia'), '宋体')
        title_run.font.size = Pt(12)

    def change_colums_value(self, row, change, columns_list):
        if row in columns_list:
            index = columns_list.index(row)
            columns_list[index] = change
        return columns_list

    def save(self, doc, path):
        doc.save(path)

    def set_cell_borders(self, cell, pt=0):
        """ Set the borders of the cell to be invisible or visible """
        # Define the border properties: Set the border to no border
        tc_pr = cell._tc.get_or_add_tcPr()
        tc_borders = OxmlElement('w:tcBorders')
        for border_name in ('top', 'left', 'right', 'bottom'):
            border = OxmlElement(f'w:{border_name}')
            border.set(qn('w:val'), 'nil')
            tc_borders.append(border)
        tc_pr.append(tc_borders)

    def insert_empty_paragraph(self, paragraph):
        paragraph.add_run()

    def insert_image_table(self, doc, image_paths, image_name, num_cols_per_table=2):
        num_images = len(image_paths)
        num_tables = (num_images + num_cols_per_table - 1) // num_cols_per_table

        for i in range(num_tables):
            num_rows = 2
            table = doc.add_table(rows=num_rows, cols=num_cols_per_table)
            table.autofit = True
            table.style = 'Table Grid'
            table.allow_autofit = True
            table.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER

            for row in table.rows:
                for cell in row.cells:
                    self.set_cell_borders(cell)

            for j in range(num_cols_per_table):
                cell_title = table.cell(0, j)
                cell_image = table.cell(1, j)

                image_index = i * num_cols_per_table + j
                if image_index < num_images:
                    self.insert_centered_title(cell_title.paragraphs[0], image_name[image_index])
                    paragraph = cell_image.paragraphs[0]
                    paragraph.alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
                    run = paragraph.add_run()
                    cell_width = table.cell(1, j).width - Cm(0.8)
                    run.add_picture(image_paths[image_index], width=cell_width)
                else:
                    self.insert_centered_title(cell_title.paragraphs[0], '')
                    self.insert_empty_paragraph(cell_image.paragraphs[0])





# SCADA故障数据DEMO
import pandas as pd

from ErrorAnalysis.error import SCADAError

# SCADA故障分析具体用法
from GeneratorPerformanceAnalysis.generator_performance_analysis import GeneratorPerformance
from PowerGenerationStatistics.power_generation_statistics import PowerGenerationStatistics
from Report.report import Report
from Utility.save_data_tool import SaveData, get_data
from Utility.utility import Utility
from YawAnalysis.YawAnalysis import YawAnalysis


def ErrorDemo():
    data = pd.read_csv('./SampleData/SampleErrorScada.csv')
    scada_error = SCADAError(data, 'event_unit','event_start_time','event_end_time','event_type','故障次数')
    scada_error.error_analysis()
    scada_error = SCADAError(data, 'event_unit','event_start_time','event_end_time','event_type','停机时间')
    scada_error.error_analysis()
    scada_error.error_statistical('故障')

# SCADA数据发电量统计分析具体用法
def PowerDemo():
    data = pd.read_csv('./SampleData/SampleScada.csv')
    power = PowerGenerationStatistics(data, 'wt_id','tal_ac_pw_gen', 'time', 'daily')
    power.power_analysis()
    power = PowerGenerationStatistics(data, 'wt_id', 'tal_ac_pw_gen', 'time', 'monthly')
    power.power_analysis()
    power = PowerGenerationStatistics(data, 'wt_id', 'tal_ac_pw_gen', 'time', 'all')
    power.power_analysis()

# SCADA数据发电性能分析具体用法
def GeneratorPerformanceDemo():
    data = pd.read_csv('./SampleData/SampleScada.csv')
    generator_performance = GeneratorPerformance(data, 'wind_speed','active_power','风速','功率','风速-功率曲线')
    generator_performance.Plot()
    generator_performance = GeneratorPerformance(data, 'generator_torque', 'wheel_rpm', '转矩', '转速', '转矩-转速曲线')
    generator_performance.Plot()
    generator_performance = GeneratorPerformance(data, 'wheel_rpm', 'blade1_angle', '转速', '桨距角', '转速-桨距角曲线')
    generator_performance.Plot()
    generator_performance = GeneratorPerformance(data, 'wind_speed', 'blade1_angle', '风速', '桨距角', '风速-桨距角曲线')
    generator_performance.Plot()
    generator_performance = GeneratorPerformance(data, 'wind_speed', 'wheel_rpm', '风速', '转速', '风速-转速曲线')
    generator_performance.Plot()
    generator_performance = GeneratorPerformance(data, 'wind_speed', 'wind_speed', '风速', '风速', '风速曲线')
    generator_performance.Plot()

# SCADA秒级数据，偏航分析
def YawAnalysisDemo():
    yaw_analysis = YawAnalysis('./SampleData/SecondData',trubine_field = '风机ID',hydraulic_pressure = '液压制动压力',angle_of_wind = '风向绝对值',
                               fan_status = '风机当前状态值',yaw_require = '偏航要求值',yaw_times = '总偏航次数',build_pressure_speed = None, angle = None)
    yaw_analysis.analysis()
    result = yaw_analysis.get_result()
    print(result)

def ReportDemo():
    report = Report('./SampleData/封面.docx')
    world = report.create_new_document()
    small_title_table_content = []
    analysis_type = {
        '发电机性能分析': ['风速-功率曲线', '转矩-转速曲线', '转速-桨距角曲线', '风速-转速曲线', '风速-桨距角曲线', '平均风速', '年平均风速', '理论电量', '实际电量']}
    functionality = "机组总体分析"
    feature_branch = "发电机性能分析"
    big_title = 1
    report.insert_title(world, str(big_title) + "、 " + "机组发电性能分析")
    small_title = 1
    flag = 0
    data = pd.DataFrame()
    _content = None
    table_flag = False
    utility = Utility()
    if '数据开始时间' not in data.columns and '数据结束时间' not in data.columns:
        data['数据开始时间'] = None
        data['数据结束时间'] = None
    for i in analysis_type['发电机性能分析']:
        save_type = SaveData(functionality, feature_branch, i)
        save_type = get_data(save_type, get_data_type="all", file_path='./SampleData/data.json')
        if not save_type:
            continue
        if i == '平均风速' or i == '年平均风速' or i == '理论电量' or i == '实际电量':
            flag += 1
            if flag == 1:
                report.insert_lettle_title_word(world,
                                                 str(big_title) + '.' + str(small_title) + "、 各风机风速和电量分析结果")
                small_title_table_content.append(str(big_title) + '.' + str(small_title) + "、 各风机风速和电量分析结果")
        else:
            report.insert_lettle_title_word(world,
                                             str(big_title) + '.' + str(small_title) + "、 各风机" + i + "分析结果")
            small_title_table_content.append(str(big_title) + '.' + str(small_title) + "、 各风机" + i + "分析结果")
        small_title += 1
        name = []
        path = []
        report_content = ''
        for id, content in save_type.items():
            if content['type'] == 'image':
                name.append(id)
                path.append(utility.resource_path(content['data'][0]).replace('\\', '/').replace('./', ''))
                report_content = report_content + content['content']
            elif content['type'] == 'table':
                table_flag = True
                if id not in data.index:
                    data.loc[id] = 0
                if i == '理论电量':
                    data.loc[id, i] = str(int(float(content['data'])))
                else:
                    data.loc[id, i] = str(content['data'])
                data.loc[id, '数据开始时间'] = content['start_time']
                data.loc[id, '数据结束时间'] = content['end_time']
                _content = content['content']
        if name:
            report.insert_image_table(world, path, name)
            report.insert_paragraph(world, report_content)
    if table_flag:
        target_columns = ['数据开始时间', '数据结束时间']
        moved_columns = [data.pop(col) for col in target_columns]
        for col in moved_columns:
            data[col.name] = col
        data.index.name = '风机'
        data_reset = data.reset_index()
        columns_list = data_reset.columns.tolist()
        columns_list = report.change_colums_value('平均风速', '平均风速(m/s)', columns_list)
        columns_list = report.change_colums_value('年平均风速', '年平均风速(m/s)', columns_list)
        columns_list = report.change_colums_value('理论电量', '理论电量(kw·h)', columns_list)
        columns_list = report.change_colums_value('实际电量', '实际电量(kw·h)', columns_list)
        values_list = data_reset.values.tolist()
        final_list = [columns_list] + [[str(val) for val in row] for row in values_list]
        final_list = utility.process_data(final_list)
        report.insert_table_word(world, final_list)
    print("机组发电性能分析报告生成成功")
    report.save(world, '报告.docx')
    # self.table_content[str(self.big_title) + "、 " + "机组发电性能分析"] = small_title_table_content
    # self.big_title += 1

if __name__ == '__main__':
    ErrorDemo()
    # PowerDemo()
    # GeneratorPerformanceDemo()
    # YawAnalysisDemo()
    # ReportDemo()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/12/5 14:04
# @Author  : 曹智辉
# @File    : save_data_tool.py
# @Description : 保存数据工具
import json
import os

class SaveData:
    def __init__(self, functionality, feature_branch, analysis_functionality, analysis_data=None, id=None, type=None,
                 content=None, comments_example=None, start_time=None, end_time=None):
        self.functionality = functionality
        self.feature_branch = feature_branch
        self.analysis_functionality = analysis_functionality
        self.analysis_data = analysis_data
        self.id = id
        self.type = type
        self.content = content
        self.comments_example = comments_example
        self.start_time = start_time
        self.end_time = end_time

    def to_json(self):
        data = {
            "功能模块": self.functionality,
            "功能分支": self.feature_branch,
            "具体功能": self.analysis_functionality,
            "分析数据": {
                self.id: {
                    "type": self.type,
                    "data": self.analysis_data,
                    "content": self.content,
                    "comments_example": self.comments_example,
                    "start_time": self.start_time,
                    "end_time": self.end_time
                }
            }
        }
        return json.dumps(data)

def save_data_to_file(file_path, data):
    with open(file_path, 'w', encoding='utf-8') as f:
        json.dump(data, f, indent=2,ensure_ascii=False)


def load_data_from_file(file_path):
    if not os.path.isfile(file_path) or os.path.getsize(file_path) == 0:
        return []
    with open(file_path, 'r', encoding='utf-8') as file:
        data = file.read()
        if data:
            return json.loads(data)
        else:
            return []

def add_data(data, file_path):
    existing_data = load_data_from_file(file_path)

    new_data = {
        "type": data.type,
        "data": data.analysis_data,
        "content": data.content,
        "comments_example": data.comments_example,
        "start_time": data.start_time,
        "end_time": data.end_time
    }

    found = False
    for item in existing_data:
        if item["功能模块"] == data.functionality and item["功能分支"] == data.feature_branch and item["具体功能"] == data.analysis_functionality:
            item["分析数据"].update({data.id: new_data})
            found = True
            break

    if not found:
        new_item = {
            "功能模块": data.functionality,
            "功能分支": data.feature_branch,
            "具体功能": data.analysis_functionality,
            "分析数据": {data.id: new_data}
        }
        existing_data.append(new_item)

    save_data_to_file(existing_data, file_path)


def delete_data(data, id, file_path):
    existing_data = data.load_data_from_file(file_path)
    if id in existing_data["分析数据"]:
        del existing_data["分析数据"][id]
        save_data_to_file(existing_data, file_path)


def update_data(data, id, updated_data, file_path):
    existing_data = load_data_from_file(file_path)

    for item in existing_data:
        if item["功能模块"] == data.functionality and item["功能分支"] == data.feature_branch and item["具体功能"] == data.analysis_functionality:
            if id in item["分析数据"]:
                item["分析数据"][id].update(updated_data)
                save_data_to_file(existing_data, file_path)
                return True

    return False


def get_data(data, file_path, data_id = None, get_data_type = None):
    existing_data = load_data_from_file(file_path)

    for item in existing_data:
        if item["功能模块"] == data.functionality and item["功能分支"] == data.feature_branch and item["具体功能"] == data.analysis_functionality:
            if get_data_type == "all":
                return item["分析数据"]
            if data_id in item["分析数据"]:
                return item["分析数据"][data_id]

    return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os
from matplotlib import pyplot as plt

plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False

class Utility:
    def __init__(self):
        pass

    def resource_path(self, relative_path):
        if getattr(sys, 'frozen', False) and 'save_data' in relative_path:
            base_path = os.path.dirname(sys.executable)
        elif hasattr(sys, '_MEIPASS'):
            base_path = sys._MEIPASS
        else:
            base_path = os.path.abspath(".")
        return os.path.join(base_path, relative_path)

    def ScatterPlot(self, x, y, title, x_label, y_label, x_min=None,x_max=None,y_min=None,y_max=None, wind_data=None):
        plt.close()
        plt.scatter(x, y, s=1, alpha=0.5)
        plt.xlabel(x_label)
        plt.ylabel(y_label)
        plt.title(title)
        if x_min:
            plt.xlim(xmin=x_min)
        if x_max:
            plt.xlim(xmax=x_max)
        if y_min:
            plt.ylim(ymin=y_min)
        if y_max:
            plt.ylim(ymax=y_max)
        plt.show()
        plt.close()

    def process_data(self, original_data):
        headers = original_data[0]
        try:
            avg_wind_speed_index = headers.index('年平均风速')
        except ValueError:
            return original_data

        new_data = [headers]
        for row in original_data[1:]:
            try:
                year_avg_speed_dict = ast.literal_eval(row[avg_wind_speed_index])
            except (ValueError, SyntaxError):
                continue

            if isinstance(year_avg_speed_dict, dict):
                for year, speed in year_avg_speed_dict.items():
                    new_row = row[:avg_wind_speed_index] + [f"{year}: {speed}"] + row[avg_wind_speed_index + 1:]
                    new_data.append(new_row)
            else:
                new_data.append(row)
        return new_data


from setuptools import setup, find_packages
import os

VERSION = '0.0.10'
DESCRIPTION = 'wind turbine ai and digital twin'

setup(
    name="wtai_dt",
    version=VERSION,
    author="hnust",
    author_email="2538134102@qq.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=open('README.md',encoding="UTF8").read(),
    packages=find_packages(),
    install_requires=['moviepy'],
    keywords=['python', 'moviepy', 'cut video'],
    # data_files=[('wtai_dt', ['to_erase.json'])],
    # entry_points={
    # 'console_scripts': [
    #     'wtai_dt = wtai_dt.main:main'
    # ]
    # },
    license="MIT",
    url="https://github.com/importimport/wtai-dt",
    scripts=['wtai_dt/wtai_dt.py'],
    classifiers= [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows"
    ]
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import datetime

import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False

colors = plt.cm.tab20.colors
def process_data(power_data):
    result_dict = {}
    for key, value in power_data.items():
        parts = key.split(' ')
        if len(parts) == 2:
            unit = parts[0]
            date = parts[1]
        else:
            unit = key
            date = None
        if unit not in result_dict:
            result_dict[unit] = {}
        if date:
            result_dict[unit][date] = value
        else:
            result_dict[unit]['全场发电量'] = value
    return result_dict

def classify_columns(col):
    try:
        datetime.strptime(col, '%Y-%m-%d')
        return 'daily'
    except ValueError:
        try:
            datetime.strptime(col, '%Y-%m')
            return 'monthly'
        except ValueError:
            return 'unknown'

def wind_power_daily(data):
    plt.figure(figsize=(14, 7))
    for column in data.columns:
        data[column] = data[column].astype(float).astype(int)
    bottom = np.zeros(len(data.columns))
    for i, wind in enumerate(data.index):
        color = colors[i % len(colors)]
        plt.bar(data.columns, data.loc[wind], color=color, label=wind, bottom=bottom)
        bottom += data.loc[wind].values
    plt.xlabel('日期')
    plt.ylabel('发电量 (万kWh)')
    plt.title('每天的发电量')
    plt.xticks(rotation=90)
    if len(data.columns) >= 20:
        ncols = len(data.columns) // 3
    elif len(data.columns) >= 10:
        ncols = len(data.columns) // 2
    else:
        ncols = len(data.columns)
    plt.legend(loc='upper center', bbox_to_anchor=(0.5, -0.15), ncol=ncols)
    plt.tight_layout()
    plt.show()

def wind_power_monthly(data):
    plt.close()
    plt.figure(figsize=(14, 7))
    months = data.columns
    wind_machines = data.index
    bar_width = 0.15
    x = np.arange(len(wind_machines))
    for i, month in enumerate(months):
        plt.bar(x + i * bar_width, data[month].astype(float), width=bar_width, label=f'{month} 月')
    plt.ylabel('发电量 (万kWh)')
    plt.title(f'各风机月发电量')
    plt.xticks(x + bar_width * (len(months) - 1) / 2, wind_machines)
    plt.xticks(rotation=90)
    plt.legend(title='月份')
    plt.tight_layout()
    plt.show()
    plt.close()

class PowerGenerationStatistics:
    def __init__(self, data, wind_field, power_generation_field,time_field,function):
        """
            初始化发电量统计分析类

            Args:
                data: 发电量数据
                wind_field: 风机字段
                time_field: 时间字段
                power_generation_field: 发电量字段
                function: 发电量函数:daily,monthly,all
        """
        self.data = data
        self.wind_field = wind_field
        self.time_field = time_field
        self.power_generation_field = power_generation_field
        self.function = function
        self.result_dict = {}

    def init_data(self):
        self.data[self.time_field] = pd.to_datetime(self.data[self.time_field])

    def day_generate(self, data, field, time):
        # 计算日发电量
        data[time] = pd.to_datetime(data[time])
        data = data.sort_values(time)
        data = data.set_index(time)
        data['day'] = data[field].diff()
        day_energy = data['day'].resample('D').sum()
        day_energy = {str(k.date()): v for k, v in day_energy.items()}  # 将日期转换为字符串格式
        print(day_energy)
        return day_energy

    def month_generate(self, data, field, time):
        # 计算月发电量
        data[time] = pd.to_datetime(data[time])
        data = data.sort_values(time)
        data = data.set_index(time)
        data['month'] = data[field].diff()
        month_energy = data['month'].resample('M').sum()
        month_energy = {k.strftime('%Y-%m'): v for k, v in month_energy.items()}  # 将日期转换为年-月格式
        print(month_energy)
        return month_energy

    def total_generate(self, data, field, time):
        data[time] = pd.to_datetime(data[time])
        data = data.sort_values(time)
        data = data.set_index(time)
        data['total'] = data[field].diff()
        total_power = data['total'].sum()
        print(total_power)
        return {'全场发电量': total_power}

    def power_analysis(self):
        d = self.data.groupby(self.wind_field)
        field = self.power_generation_field
        time = self.time_field
        for group_name, group_data in d:
            if self.function == 'daily':
                result_dict = self.day_generate(group_data, field, time)
            elif self.function == 'monthly':
                result_dict = self.month_generate(group_data,field,time)
            elif self.function == 'all':
                result_dict = self.total_generate(group_data,field,time)
            else:
                print("请选择正确的发电量计算函数:daily,monthly,all")
                return
            temp_dict = {}
            for key, value in result_dict.items():
                result_key = "风机" + str(group_name) + " " + key
                temp_dict.setdefault(result_key, str(value))
                self.result_dict[result_key] = str(value)

    def plot_image(self):
        result = process_data(self.result_dict)
        data = pd.DataFrame(result).T
        data = data.fillna(0)
        try:
            data = data.drop(columns=['全场发电量'])
            data = data.drop(index=['全场发电量'])
        except Exception as e:
            return e
        classification = {col: classify_columns(col) for col in data.columns}
        monthly_df , daily_df = None, None
        try:
            monthly_df = data[[col for col, clas in classification.items() if clas == 'monthly']]
            daily_df = data[[col for col, clas in classification.items() if clas == 'daily']]
        except Exception as e:
            print(e)
        if monthly_df is not None:
            wind_power_monthly(monthly_df)
            print('各机组月发电量画图完成')
            print('各机组月发电量数据提取成功')
        else:
            print('无法提取各机组月发电量数据')

        if daily_df is not None:
            wind_power_daily(daily_df)
            print('每天的发电量画图完成')
            print('每天的发电量数据提取成功')
        else:
            print('无法提取每天的发电量数据')